@@ -41,6 +41,11 @@
 #include "Command.h"
 #include "DownloadEngine.h"
 #include "DownloadContext.h"
+#include "FileEntry.h"
+#include "PieceStorage.h"
+#include "DiskAdaptor.h"
+#include "Option.h"
+#include "prefs.h"
 
 namespace aria2 {
 
@@ -52,13 +57,20 @@ BtFileAllocationEntry::~BtFileAllocationEntry() {}
 void BtFileAllocationEntry::prepareForNextAction
 (std::vector<Command*>& commands, DownloadEngine* e)
 {
-  BtSetup().setup(commands, _requestGroup, e, _requestGroup->getOption().get());
-  if(!_requestGroup->downloadFinished()) {
-    _requestGroup->getDownloadContext()->resetDownloadStartTime();
+  BtSetup().setup(commands, getRequestGroup(), e,
+                  getRequestGroup()->getOption().get());
+  if(getRequestGroup()->getOption()->getAsBool(PREF_ENABLE_MMAP)) {
+    getRequestGroup()->getPieceStorage()->getDiskAdaptor()->enableMmap();
+  }
+  if(!getRequestGroup()->downloadFinished()) {
+    // For DownloadContext::resetDownloadStartTime(), see also
+    // RequestGroup::createInitialCommand()
+    getRequestGroup()->getDownloadContext()->resetDownloadStartTime();
     const std::vector<SharedHandle<FileEntry> >& fileEntries =
-      _requestGroup->getDownloadContext()->getFileEntries();
-    if(isUriSuppliedForRequsetFileEntry(fileEntries.begin(), fileEntries.end())) {
-      _requestGroup->createNextCommandWithAdj(commands, e, 0);
+      getRequestGroup()->getDownloadContext()->getFileEntries();
+    if(isUriSuppliedForRequsetFileEntry
+       (fileEntries.begin(), fileEntries.end())) {
+      getRequestGroup()->createNextCommandWithAdj(commands, e, 0);
     }
   }
 }
