@@ -50,14 +50,15 @@ class BtAnnounce;
 class BtRuntime;
 class BtProgressInfoFile;
 class DownloadContext;
+class LpdMessageReceiver;
 
 struct BtObject {
-  SharedHandle<DownloadContext> downloadContext_;
-  SharedHandle<PieceStorage> pieceStorage_;
-  SharedHandle<PeerStorage> peerStorage_;
-  SharedHandle<BtAnnounce> btAnnounce_;
-  SharedHandle<BtRuntime> btRuntime_;
-  SharedHandle<BtProgressInfoFile> btProgressInfoFile_;
+  SharedHandle<DownloadContext> downloadContext;
+  SharedHandle<PieceStorage> pieceStorage;
+  SharedHandle<PeerStorage> peerStorage;
+  SharedHandle<BtAnnounce> btAnnounce;
+  SharedHandle<BtRuntime> btRuntime;
+  SharedHandle<BtProgressInfoFile> btProgressInfoFile;
 
   BtObject(const SharedHandle<DownloadContext>& downloadContext,
            const SharedHandle<PieceStorage>& pieceStorage,
@@ -73,30 +74,33 @@ struct BtObject {
   ~BtObject();
 
   BtObject& operator=(const BtObject& c);
-
-  bool isNull() const;
 };
 
 class BtRegistry {
 private:
-  std::map<a2_gid_t, BtObject> pool_;
+  std::map<a2_gid_t, SharedHandle<BtObject> > pool_;
+  uint16_t tcpPort_;
+  SharedHandle<LpdMessageReceiver> lpdMessageReceiver_;
 public:
-  SharedHandle<DownloadContext>
+  BtRegistry();
+  ~BtRegistry();
+
+  const SharedHandle<DownloadContext>&
   getDownloadContext(a2_gid_t gid) const;
 
-  SharedHandle<DownloadContext>
+  const SharedHandle<DownloadContext>&
   getDownloadContext(const std::string& infoHash) const;
 
-  void put(a2_gid_t gid, const BtObject& obj);
+  void put(a2_gid_t gid, const SharedHandle<BtObject>& obj);
 
-  BtObject get(a2_gid_t gid) const;
+  const SharedHandle<BtObject>& get(a2_gid_t gid) const;
 
   template<typename OutputIterator>
   OutputIterator getAllDownloadContext(OutputIterator dest)
   {
-    for(std::map<a2_gid_t, BtObject>::const_iterator i = pool_.begin(),
-          eoi = pool_.end(); i != eoi; ++i) {
-      *dest++ = (*i).second.downloadContext_;
+    for(std::map<a2_gid_t, SharedHandle<BtObject> >::const_iterator i =
+          pool_.begin(), eoi = pool_.end(); i != eoi; ++i) {
+      *dest++ = (*i).second->downloadContext;
     }
     return dest;
   }
@@ -104,6 +108,21 @@ public:
   void removeAll();
 
   bool remove(a2_gid_t gid);
+
+  void setTcpPort(uint16_t port)
+  {
+    tcpPort_ = port;
+  }
+  uint16_t getTcpPort() const
+  {
+    return tcpPort_;
+  }
+
+  void setLpdMessageReceiver(const SharedHandle<LpdMessageReceiver>& receiver);
+  const SharedHandle<LpdMessageReceiver>& getLpdMessageReceiver() const
+  {
+    return lpdMessageReceiver_;
+  }
 };
 
 } // namespace aria2
