@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_DISK_WRITER_H_
-#define _D_DISK_WRITER_H_
+#ifndef D_DISK_WRITER_H
+#define D_DISK_WRITER_H
 
 #include "BinaryStream.h"
 
@@ -44,21 +44,16 @@ namespace aria2 {
  *
  */
 class DiskWriter:public BinaryStream {
-protected:
-  bool _directIOAllowed;
-
 public:
-  DiskWriter():_directIOAllowed(false) {}
+  DiskWriter() {}
 
   virtual ~DiskWriter() {}
   /**
-   * Creates a file output stream to write to the file with the specified name.
-   * If the file exists, then it is truncated to 0 length.
-   * @param filename the file name to be opened.
+   * Opens file. If the file exists, then it is truncated to 0 length.
    */
-  virtual void initAndOpenFile(const std::string& filename, uint64_t totalLength = 0)  = 0;
+  virtual void initAndOpenFile(int64_t totalLength = 0)  = 0;
   
-  virtual void openFile(const std::string& filename, uint64_t totalLength = 0) = 0;
+  virtual void openFile(int64_t totalLength = 0) = 0;
 
   /**
    * Closes this output stream.
@@ -67,29 +62,13 @@ public:
   virtual void closeFile() = 0;
 
   /**
-   * Opens a file output stream to write to the file with the specified name.
-   * If the file doesnot exists, an exception may be throwed.
-   *
-   * @param filename the file name to be opened.
+   * Opens a file.  If the file doesnot exists, an exception may be
+   * thrown.
    */
-  virtual void openExistingFile(const std::string& filename, uint64_t totalLength = 0) = 0;
+  virtual void openExistingFile(int64_t totalLength = 0) = 0;
 
   // Returns file length
-  virtual uint64_t size() = 0;
-
-  virtual void enableDirectIO() = 0;
-
-  virtual void disableDirectIO() = 0;
-
-  virtual bool directIOAllowed() const
-  {
-    return _directIOAllowed;
-  }
-
-  void setDirectIOAllowed(bool f)
-  {
-    _directIOAllowed = f;
-  }
+  virtual int64_t size() = 0;
 
   // Enables read-only mode. After this call, openExistingFile() opens
   // file in read-only mode. This is an optional functionality. The
@@ -100,10 +79,13 @@ public:
   // opens file in read/write mode. This is an optional
   // functionality. The default implementation is do noting.
   virtual void disableReadOnly() {}
+
+  // Enables mmap.
+  virtual void enableMmap() {}
 };
 
 typedef SharedHandle<DiskWriter> DiskWriterHandle;
 
 } // namespace aria2
 
-#endif // _D_DISK_WRITER_H_
+#endif // D_DISK_WRITER_H
