@@ -45,14 +45,14 @@
 #include "TimerA2.h"
 #include "A2STR.h"
 #include "ValueBase.h"
-#include "IntSequence.h"
+#include "SegList.h"
 
 namespace aria2 {
 
 class RequestGroup;
 class Signature;
 class FileEntry;
-class ContextAttribute;
+struct ContextAttribute;
 
 class DownloadContext
 {
@@ -61,13 +61,13 @@ private:
 
   std::vector<std::string> pieceHashes_;
 
-  size_t pieceLength_;
+  int32_t pieceLength_;
 
-  std::string pieceHashAlgo_;
+  std::string pieceHashType_;
 
-  std::string checksum_;
+  std::string digest_;
 
-  std::string checksumHashAlgo_;
+  std::string hashType_;
 
   bool checksumVerified_;
 
@@ -84,13 +84,16 @@ private:
   Timer downloadStopTime_;
 
   SharedHandle<Signature> signature_;
+  // This member variable is required to avoid to parse Metalink/HTTP
+  // Link header fields multiple times.
+  bool metalinkServerContacted_;
 public:
   DownloadContext();
 
   // Convenient constructor that creates single file download.  path
   // should be escaped with util::escapePath(...).
-  DownloadContext(size_t pieceLength,
-                  uint64_t totalLength,
+  DownloadContext(int32_t pieceLength,
+                  int64_t totalLength,
                   const std::string& path = A2STR::NIL);
 
   ~DownloadContext();
@@ -103,12 +106,15 @@ public:
   }
 
   template<typename InputIterator>
-  void setPieceHashes(InputIterator first, InputIterator last)
+  void setPieceHashes
+  (const std::string& hashType,
+   InputIterator first, InputIterator last)
   {
+    pieceHashType_ = hashType;
     pieceHashes_.assign(first, last);
   }
 
-  uint64_t getTotalLength() const;
+  int64_t getTotalLength() const;
 
   bool knowsTotalLength() const { return knowsTotalLength_; }
 
@@ -140,23 +146,19 @@ public:
     fileEntries_.assign(first, last);
   }
 
-  size_t getPieceLength() const { return pieceLength_; }
+  int32_t getPieceLength() const { return pieceLength_; }
 
-  void setPieceLength(size_t length) { pieceLength_ = length; }
+  void setPieceLength(int32_t length) { pieceLength_ = length; }
 
   size_t getNumPieces() const;
 
-  const std::string& getPieceHashAlgo() const { return pieceHashAlgo_; }
+  const std::string& getPieceHashType() const { return pieceHashType_; }
 
-  void setPieceHashAlgo(const std::string& algo);
+  const std::string& getDigest() const { return digest_; }
 
-  const std::string& getChecksum() const { return checksum_; }
+  const std::string& getHashType() const { return hashType_; }
 
-  void setChecksum(const std::string& checksum);
-
-  const std::string& getChecksumHashAlgo() const { return checksumHashAlgo_; }
-
-  void setChecksumHashAlgo(const std::string& algo);
+  void setDigest(const std::string& hashType, const std::string& digest);
 
   // The representative path name for this context. It is used as a
   // part of .aria2 control file. If basePath_ is set, returns
@@ -177,13 +179,12 @@ public:
     ownerRequestGroup_ = owner;
   }
 
-  void setFileFilter(IntSequence seq);
+  // sgl must be normalized before the call.
+  void setFileFilter(SegList<int>& sgl);
 
   // Sets file path for specified index. index starts from 1. The
-  // index is the same used in setFileFilter().  Please note that path
-  // is not the actual file path. The actual file path is
-  // getDir()+"/"+path. path is not escaped by util::escapePath() in
-  // this function.
+  // index is the same used in setFileFilter(). path is not escaped by
+  // util::escapePath() in this function.
   void setFilePathWithIndex(size_t index, const std::string& path);
 
   // Returns true if hash check(whole file hash, not piece hash) is
@@ -221,9 +222,18 @@ public:
   
   // Returns FileEntry at given offset. SharedHandle<FileEntry>() is
   // returned if no such FileEntry is found.
-  SharedHandle<FileEntry> findFileEntryByOffset(off_t offset) const;
+  SharedHandle<FileEntry> findFileEntryByOffset(int64_t offset) const;
 
   void releaseRuntimeResource();
+
+  void setMetalinkServerContacted(bool f)
+  {
+    metalinkServerContacted_ = f;
+  }
+  bool getMetalinkServerContacted() const
+  {
+    return metalinkServerContacted_;
+  }
 };
 
 } // namespace aria2
