@@ -56,7 +56,8 @@
 #include "Checksum.h"
 #include "uri.h"
 #include "MetalinkHttpEntry.h"
-#include "Base64.h"
+#include "base64.h"
+#include "array_fun.h"
 #ifdef ENABLE_MESSAGE_DIGEST
 #include "MessageDigest.h"
 #endif // ENABLE_MESSAGE_DIGEST
@@ -99,12 +100,12 @@ void HttpResponse::validateResponse() const
       if(!httpRequest_->isRangeSatisfied(responseRange)) {
         throw DL_ABORT_EX2
           (fmt(EX_INVALID_RANGE_HEADER,
-               util::itos(httpRequest_->getStartByte(), true).c_str(),
-               util::itos(httpRequest_->getEndByte(), true).c_str(),
-               util::uitos(httpRequest_->getEntityLength(), true).c_str(),
-               util::itos(responseRange->getStartByte(), true).c_str(),
-               util::itos(responseRange->getEndByte(), true).c_str(),
-               util::uitos(responseRange->getEntityLength(), true).c_str()),
+               httpRequest_->getStartByte(),
+               httpRequest_->getEndByte(),
+               httpRequest_->getEntityLength(),
+               responseRange->getStartByte(),
+               responseRange->getEndByte(),
+               responseRange->getEntityLength()),
            error_code::CANNOT_RESUME);
       }
     }
@@ -118,9 +119,11 @@ std::string HttpResponse::determinFilename() const
 {
   std::string contentDisposition =
     util::getContentDispositionFilename
-    (httpHeader_->getFirst(HttpHeader::CONTENT_DISPOSITION));
+    (httpHeader_->find(HttpHeader::CONTENT_DISPOSITION));
   if(contentDisposition.empty()) {
-    std::string file = util::percentDecode(httpRequest_->getFile());
+    std::string file =
+      util::percentDecode(httpRequest_->getFile().begin(),
+                          httpRequest_->getFile().end());
     if(file.empty()) {
       return "index.html";
     } else {
@@ -139,7 +142,7 @@ void HttpResponse::retrieveCookie()
   Time now;
   std::pair<std::multimap<std::string, std::string>::const_iterator,
             std::multimap<std::string, std::string>::const_iterator> r =
-    httpHeader_->getIterator(HttpHeader::SET_COOKIE);
+    httpHeader_->equalRange(HttpHeader::SET_COOKIE);
   for(; r.first != r.second; ++r.first) {
     httpRequest_->getCookieStorage()->parseAndStore
       ((*r.first).second, httpRequest_->getHost(), httpRequest_->getDir(),
@@ -167,7 +170,7 @@ void HttpResponse::processRedirect()
                     httpRequest_->getRequest()->getCurrentUri().c_str()));
   } else {
     throw DL_RETRY_EX
-      (fmt("CUID#%lld - Redirect to %s failed. It may not be a valid URI.",
+      (fmt("CUID#%" PRId64 " - Redirect to %s failed. It may not be a valid URI.",
            cuid_,
            httpRequest_->getRequest()->getCurrentUri().c_str()));
   }
@@ -175,7 +178,7 @@ void HttpResponse::processRedirect()
 
 const std::string& HttpResponse::getRedirectURI() const
 {
-  return httpHeader_->getFirst(HttpHeader::LOCATION);
+  return httpHeader_->find(HttpHeader::LOCATION);
 }
 
 bool HttpResponse::isTransferEncodingSpecified() const
@@ -186,7 +189,7 @@ bool HttpResponse::isTransferEncodingSpecified() const
 const std::string& HttpResponse::getTransferEncoding() const
 {
   // TODO See TODO in getTransferEncodingStreamFilter()
-  return httpHeader_->getFirst(HttpHeader::TRANSFER_ENCODING);
+  return httpHeader_->find(HttpHeader::TRANSFER_ENCODING);
 }
 
 SharedHandle<StreamFilter> HttpResponse::getTransferEncodingStreamFilter() const
@@ -209,7 +212,7 @@ bool HttpResponse::isContentEncodingSpecified() const
 
 const std::string& HttpResponse::getContentEncoding() const
 {
-  return httpHeader_->getFirst(HttpHeader::CONTENT_ENCODING);
+  return httpHeader_->find(HttpHeader::CONTENT_ENCODING);
 }
 
 SharedHandle<StreamFilter> HttpResponse::getContentEncodingStreamFilter() const
@@ -224,7 +227,7 @@ SharedHandle<StreamFilter> HttpResponse::getContentEncodingStreamFilter() const
   return filter;
 }
 
-uint64_t HttpResponse::getContentLength() const
+int64_t HttpResponse::getContentLength() const
 {
   if(!httpHeader_) {
     return 0;
@@ -233,7 +236,7 @@ uint64_t HttpResponse::getContentLength() const
   }
 }
 
-uint64_t HttpResponse::getEntityLength() const
+int64_t HttpResponse::getEntityLength() const
 {
   if(!httpHeader_) {
     return 0;
@@ -247,9 +250,10 @@ std::string HttpResponse::getContentType() const
   if(!httpHeader_) {
     return A2STR::NIL;
   } else {
-    std::pair<std::string, std::string> p;
-    util::divide(p, httpHeader_->getFirst(HttpHeader::CONTENT_TYPE), ';');
-    return p.first;
+    const std::string& ctype = httpHeader_->find(HttpHeader::CONTENT_TYPE);
+    std::string::const_iterator i = std::find(ctype.begin(), ctype.end(), ';');
+    Scip p = util::stripIter(ctype.begin(), i);
+    return std::string(p.first, p.second);
   }
 }
 
@@ -275,27 +279,35 @@ bool HttpResponse::hasRetryAfter() const
 
 time_t HttpResponse::getRetryAfter() const
 {
-  return httpHeader_->getFirstAsUInt(HttpHeader::RETRY_AFTER);
+  return httpHeader_->findAsInt(HttpHeader::RETRY_AFTER);
 }
 
 Time HttpResponse::getLastModifiedTime() const
 {
-  return Time::parseHTTPDate(httpHeader_->getFirst(HttpHeader::LAST_MODIFIED));
+  return Time::parseHTTPDate(httpHeader_->find(HttpHeader::LAST_MODIFIED));
 }
 
 bool HttpResponse::supportsPersistentConnection() const
 {
-  std::string connection =
-    util::toLower(httpHeader_->getFirst(HttpHeader::CONNECTION));
-  std::string version = httpHeader_->getVersion();
-
+  const std::string& connection = httpHeader_->find(HttpHeader::CONNECTION);
+  const std::string& version = httpHeader_->getVersion();
+  const std::string& proxyConn =
+    httpHeader_->find(HttpHeader::PROXY_CONNECTION);
   return
-    connection.find(HttpHeader::CLOSE) == std::string::npos &&
+    util::strifind(connection.begin(),
+                   connection.end(),
+                   HttpHeader::CLOSE.begin(),
+                   HttpHeader::CLOSE.end()) == connection.end() &&
     (version == HttpHeader::HTTP_1_1 ||
-     connection.find("keep-alive") != std::string::npos) &&
+     util::strifind(connection.begin(),
+                    connection.end(),
+                    HttpHeader::KEEP_ALIVE.begin(),
+                    HttpHeader::KEEP_ALIVE.end()) != connection.end()) &&
     (!httpRequest_->isProxyRequestSet() ||
-     util::toLower(httpHeader_->getFirst("Proxy-Connection")).find("keep-alive")
-     != std::string::npos);
+     util::strifind(proxyConn.begin(),
+                    proxyConn.end(),
+                    HttpHeader::KEEP_ALIVE.begin(),
+                    HttpHeader::KEEP_ALIVE.end()) != proxyConn.end());
 }
 
 namespace {
@@ -309,11 +321,12 @@ bool parseMetalinkHttpLink(MetalinkHttpEntry& result, const std::string& s)
   if(last == s.end()) {
     return false;
   }
-  std::string uri = util::stripIter(first+1, last);
-  if(uri.empty()) {
+  std::pair<std::string::const_iterator,
+            std::string::const_iterator> p = util::stripIter(first+1, last);
+  if(p.first == p.second) {
     return false;
   } else {
-    result.uri = uri;
+    result.uri.assign(p.first, p.second);
   }
   last = std::find(last, s.end(), ';');
   if(last != s.end()) {
@@ -364,7 +377,7 @@ void HttpResponse::getMetalinKHttpEntries
 {
   std::pair<std::multimap<std::string, std::string>::const_iterator,
             std::multimap<std::string, std::string>::const_iterator> p =
-    httpHeader_->getIterator(HttpHeader::LINK);
+    httpHeader_->equalRange(HttpHeader::LINK);
   for(; p.first != p.second; ++p.first) {
     MetalinkHttpEntry e;
     if(parseMetalinkHttpLink(e, (*p.first).second)) {
@@ -374,8 +387,12 @@ void HttpResponse::getMetalinKHttpEntries
   if(!result.empty()) {
     std::vector<std::string> locs;
     if(option->defined(PREF_METALINK_LOCATION)) {
-      util::split(util::toLower(option->get(PREF_METALINK_LOCATION)),
-                  std::back_inserter(locs), ",", true);
+      const std::string& loc = option->get(PREF_METALINK_LOCATION);
+      util::split(loc.begin(), loc.end(), std::back_inserter(locs), ',', true);
+      for(std::vector<std::string>::iterator i = locs.begin(), eoi = locs.end();
+          i != eoi; ++i) {
+        util::lowercase(*i);
+      }
     }
     for(std::vector<MetalinkHttpEntry>::iterator i = result.begin(),
           eoi = result.end(); i != eoi; ++i) {
@@ -395,7 +412,7 @@ void HttpResponse::getDigest(std::vector<Checksum>& result) const
   using std::swap;
   std::pair<std::multimap<std::string, std::string>::const_iterator,
             std::multimap<std::string, std::string>::const_iterator> p =
-    httpHeader_->getIterator(HttpHeader::DIGEST);
+    httpHeader_->equalRange(HttpHeader::DIGEST);
   for(; p.first != p.second; ++p.first) {
     const std::string& s = (*p.first).second;
     std::string::const_iterator itr = s.begin();
@@ -408,11 +425,12 @@ void HttpResponse::getDigest(std::vector<Checksum>& result) const
         break;
       }
       util::lowercase(hashType);
-      std::string hexDigest = util::toHex(Base64::decode(digest));
-      if(!MessageDigest::isValidHash(hashType, hexDigest)) {
+      digest = base64::decode(digest.begin(), digest.end());
+      if(!MessageDigest::supports(hashType) ||
+         MessageDigest::getDigestLength(hashType) != digest.size()) {
         continue;
       }
-      result.push_back(Checksum(hashType, hexDigest));
+      result.push_back(Checksum(hashType, digest));
     }
   }
   std::sort(result.begin(), result.end(), HashTypeStronger());
