@@ -50,6 +50,10 @@
 #include "SocketRecvBuffer.h"
 #include "TimeA2.h"
 #include "array_fun.h"
+#include "JsonDiskWriter.h"
+#ifdef ENABLE_XML_RPC
+#  include "XmlRpcDiskWriter.h"
+#endif // ENABLE_XML_RPC
 
 namespace aria2 {
 
@@ -60,7 +64,11 @@ HttpServer::HttpServer
    socketRecvBuffer_(new SocketRecvBuffer(socket_)),
    socketBuffer_(socket),
    e_(e),
-   headerProcessor_(new HttpHeaderProcessor()),
+   headerProcessor_(new HttpHeaderProcessor
+                    (HttpHeaderProcessor::SERVER_PARSER)),
+   lastContentLength_(0),
+   bodyConsumed_(0),
+   reqType_(RPC_TYPE_NONE),
    keepAlive_(true),
    gzip_(false),
    acceptsPersistentConnection_(true),
@@ -69,6 +77,58 @@ HttpServer::HttpServer
 
 HttpServer::~HttpServer() {}
 
+namespace {
+const char* getStatusString(int status)
+{
+  switch(status) {
+  case 100: return "100 Continue";
+  case 101: return "101 Switching Protocols";
+  case 200: return "200 OK";
+  case 201: return "201 Created";
+  case 202: return "202 Accepted";
+  case 203: return "203 Non-Authoritative Information";
+  case 204: return "204 No Content";
+  case 205: return "205 Reset Content";
+  case 206: return "206 Partial Content";
+  case 300: return "300 Multiple Choices";
+  case 301: return "301 Moved Permanently";
+  case 302: return "302 Found";
+  case 303: return "303 See Other";
+  case 304: return "304 Not Modified";
+  case 305: return "305 Use Proxy";
+    // case 306: return "306 (Unused)";
+  case 307: return "307 Temporary Redirect";
+  case 400: return "400 Bad Request";
+  case 401: return "401 Unauthorized";
+  case 402: return "402 Payment Required";
+  case 403: return "403 Forbidden";
+  case 404: return "404 Not Found";
+  case 405: return "405 Method Not Allowed";
+  case 406: return "406 Not Acceptable";
+  case 407: return "407 Proxy Authentication Required";
+  case 408: return "408 Request Timeout";
+  case 409: return "409 Conflict";
+  case 410: return "410 Gone";
+  case 411: return "411 Length Required";
+  case 412: return "412 Precondition Failed";
+  case 413: return "413 Request Entity Too Large";
+  case 414: return "414 Request-URI Too Long";
+  case 415: return "415 Unsupported Media Type";
+  case 416: return "416 Requested Range Not Satisfiable";
+  case 417: return "417 Expectation Failed";
+    // RFC 2817 defines 426 status code.
+  case 426: return "426 Upgrade Required";
+  case 500: return "500 Internal Server Error";
+  case 501: return "501 Not Implemented";
+  case 502: return "502 Bad Gateway";
+  case 503: return "503 Service Unavailable";
+  case 504: return "504 Gateway Timeout";
+  case 505: return "505 HTTP Version Not Supported";
+  default: return "";
+  }
+}
+} // namespace
+
 SharedHandle<HttpHeader> HttpServer::receiveRequest()
 {
   if(socketRecvBuffer_->bufferEmpty()) {
@@ -77,18 +137,18 @@ SharedHandle<HttpHeader> HttpServer::receiveRequest()
       throw DL_ABORT_EX(EX_EOF_FROM_PEER);
     }
   }
-  headerProcessor_->update(socketRecvBuffer_->getBuffer(),
-                           socketRecvBuffer_->getBufferLength());
-  if(headerProcessor_->eoh()) {
-    SharedHandle<HttpHeader> header = headerProcessor_->getHttpRequestHeader();
-    size_t putbackDataLength = headerProcessor_->getPutBackDataLength();
+  SharedHandle<HttpHeader> header;
+  if(headerProcessor_->parse(socketRecvBuffer_->getBuffer(),
+                             socketRecvBuffer_->getBufferLength())) {
+    header = headerProcessor_->getResult();
     A2_LOG_INFO(fmt("HTTP Server received request\n%s",
                     headerProcessor_->getHeaderString().c_str()));
-    socketRecvBuffer_->shiftBuffer
-      (socketRecvBuffer_->getBufferLength()-putbackDataLength);
+    socketRecvBuffer_->shiftBuffer(headerProcessor_->getLastBytesProcessed());
     lastRequestHeader_ = header;
-    lastBody_.clear();
-    lastBody_.str("");
+    bodyConsumed_ = 0;
+    if(setupResponseRecv() < 0) {
+      A2_LOG_INFO("Request path is invaild. Ignore the request body.");
+    }
     lastContentLength_ =
       lastRequestHeader_->findAsLLInt(HttpHeader::CONTENT_LENGTH);
     if(lastContentLength_ < 0) {
@@ -123,16 +183,15 @@ SharedHandle<HttpHeader> HttpServer::receiveRequest()
         break;
       }
     }
-    return header;
   } else {
-    socketRecvBuffer_->clearBuffer();
-    return SharedHandle<HttpHeader>();
+    socketRecvBuffer_->shiftBuffer(headerProcessor_->getLastBytesProcessed());
   }
+  return header;
 }
 
 bool HttpServer::receiveBody()
 {
-  if(lastContentLength_ == 0) {
+  if(lastContentLength_ == bodyConsumed_) {
     return true;
   }
   if(socketRecvBuffer_->bufferEmpty()) {
@@ -143,16 +202,13 @@ bool HttpServer::receiveBody()
   }
   size_t length =
     std::min(socketRecvBuffer_->getBufferLength(),
-             static_cast<size_t>(lastContentLength_-lastBody_.tellg()));
-  lastBody_.write(reinterpret_cast<const char*>(socketRecvBuffer_->getBuffer()),
-                  length);
+             static_cast<size_t>(lastContentLength_ - bodyConsumed_));
+  if(lastBody_) {
+    lastBody_->writeData(socketRecvBuffer_->getBuffer(), length, 0);
+  }
   socketRecvBuffer_->shiftBuffer(length);
-  return lastContentLength_ == lastBody_.tellp();
-}
-
-std::string HttpServer::getBody() const
-{
-  return lastBody_.str();
+  bodyConsumed_ += length;
+  return lastContentLength_ == bodyConsumed_;
 }
 
 const std::string& HttpServer::getMethod() const
@@ -167,51 +223,61 @@ const std::string& HttpServer::getRequestPath() const
 
 void HttpServer::feedResponse(std::string& text, const std::string& contentType)
 {
-  feedResponse("200 OK", "", text, contentType);
+  feedResponse(200, "", text, contentType);
 }
 
-void HttpServer::feedResponse(const std::string& status,
+void HttpServer::feedResponse(int status,
                               const std::string& headers,
-                              std::string& text,
+                              const std::string& text,
                               const std::string& contentType)
 {
   std::string httpDate = Time().toHTTPDate();
   std::string header= fmt("HTTP/1.1 %s\r\n"
                           "Date: %s\r\n"
-                          "Content-Type: %s\r\n"
                           "Content-Length: %lu\r\n"
                           "Expires: %s\r\n"
-                          "Cache-Control: no-cache\r\n"
-                          "%s%s",
-                          status.c_str(),
+                          "Cache-Control: no-cache\r\n",
+                          getStatusString(status),
                           httpDate.c_str(),
-                          contentType.c_str(),
                           static_cast<unsigned long>(text.size()),
-                          httpDate.c_str(),
-                          supportsGZip() ?
-                          "Content-Encoding: gzip\r\n" : "",
-                          !supportsPersistentConnection() ?
-                          "Connection: close\r\n" : "");
+                          httpDate.c_str());
+  if(!contentType.empty()) {
+    header += "Content-Type: ";
+    header += contentType;
+    header += "\r\n";
+  }
   if(!allowOrigin_.empty()) {
     header += "Access-Control-Allow-Origin: ";
     header += allowOrigin_;
     header += "\r\n";
   }
-  if(!headers.empty()) {
-    header += headers;
-    if(headers.size() < 2 ||
-       (headers[headers.size()-2] != '\r' &&
-        headers[headers.size()-1] != '\n')) {
-      header += "\r\n";
-    }
+  if(supportsGZip()) {
+    header += "Content-Encoding: gzip\r\n";
   }
-
+  if(!supportsPersistentConnection()) {
+    header += "Connection: close\r\n";
+  }
+  header += headers;
   header += "\r\n";
   A2_LOG_DEBUG(fmt("HTTP Server sends response:\n%s", header.c_str()));
   socketBuffer_.pushStr(header);
   socketBuffer_.pushStr(text);
 }
 
+void HttpServer::feedUpgradeResponse(const std::string& protocol,
+                                     const std::string& headers)
+{
+  std::string header= fmt("HTTP/1.1 101 Switching Protocols\r\n"
+                          "Upgrade: %s\r\n"
+                          "Connection: Upgrade\r\n"
+                          "%s"
+                          "\r\n",
+                          protocol.c_str(),
+                          headers.c_str());
+  A2_LOG_DEBUG(fmt("HTTP Server sends upgrade response:\n%s", header.c_str()));
+  socketBuffer_.pushStr(header);
+}
+
 ssize_t HttpServer::sendResponse()
 {
   return socketBuffer_.send();
@@ -254,4 +320,76 @@ void HttpServer::setUsernamePassword
   password_ = password;
 }
 
+int HttpServer::setupResponseRecv()
+{
+  std::string path = createPath();
+  if(getMethod() == "GET") {
+    if(path == "/jsonrpc") {
+      reqType_ = RPC_TYPE_JSONP;
+      lastBody_.reset();
+      return 0;
+    }
+  } else if(getMethod() == "POST") {
+    if(path == "/jsonrpc") {
+      if(reqType_ != RPC_TYPE_JSON) {
+        reqType_ = RPC_TYPE_JSON;
+        lastBody_.reset(new json::JsonDiskWriter());
+      }
+      return 0;
+    }
+#ifdef ENABLE_XML_RPC
+    if(path == "/rpc") {
+      if(reqType_ != RPC_TYPE_XML) {
+        reqType_ = RPC_TYPE_XML;
+        lastBody_.reset(new rpc::XmlRpcDiskWriter());
+      }
+      return 0;
+    }
+#endif // ENABLE_XML_RPC
+  }
+  reqType_ = RPC_TYPE_NONE;
+  lastBody_.reset();
+  return -1;
+}
+
+std::string HttpServer::createPath() const
+{
+  std::string reqPath = getRequestPath();
+  size_t i;
+  size_t len = reqPath.size();
+  for(i = 0; i < len; ++i) {
+    if(reqPath[i] == '#' || reqPath[i] == '?') {
+      break;
+    }
+  }
+  reqPath = reqPath.substr(0, i);
+  if(reqPath.empty()) {
+    reqPath = "/";
+  }
+  return reqPath;
+}
+
+std::string HttpServer::createQuery() const
+{
+  std::string reqPath = getRequestPath();
+  size_t i;
+  size_t len = reqPath.size();
+  for(i = 0; i < len; ++i) {
+    if(reqPath[i] == '#' || reqPath[i] == '?') {
+      break;
+    }
+  }
+  if(i == len || reqPath[i] == '#') {
+    return "";
+  } else {
+    size_t start = i;
+    for(; i < len; ++i) {
+      if(reqPath[i] == '#') {
+        break;
+      }
+    }
+    return reqPath.substr(start, i - start);
+  }
+}
+
 } // namespace aria2
