@@ -55,7 +55,14 @@ TLSContext::TLSContext()
     A2_LOG_ERROR(fmt("SSL_CTX_new() failed. Cause: %s",
                      ERR_error_string(ERR_get_error(), 0)));
   }
+  /* Disable SSLv2 and enable all workarounds for buggy servers */
+  SSL_CTX_set_options(sslCtx_, SSL_OP_ALL|SSL_OP_NO_SSLv2);
   SSL_CTX_set_mode(sslCtx_, SSL_MODE_AUTO_RETRY);
+  #ifdef SSL_MODE_RELEASE_BUFFERS
+  /* keep memory usage low */
+  SSL_CTX_set_mode(sslCtx_, SSL_MODE_RELEASE_BUFFERS);
+  #endif
+  
 }
 
 TLSContext::~TLSContext()
@@ -95,6 +102,18 @@ bool TLSContext::addClientKeyFile(const std::string& certfile,
   return true;
 }
 
+bool TLSContext::addSystemTrustedCACerts()
+{
+  if(SSL_CTX_set_default_verify_paths(sslCtx_) != 1) {
+    A2_LOG_ERROR(fmt(MSG_LOADING_SYSTEM_TRUSTED_CA_CERTS_FAILED,
+                     ERR_error_string(ERR_get_error(), 0)));
+    return false;
+  } else {
+    A2_LOG_INFO("System trusted CA certificates were successfully added.");
+    return true;
+  }
+}
+
 bool TLSContext::addTrustedCACertFile(const std::string& certfile)
 {
   if(SSL_CTX_load_verify_locations(sslCtx_, certfile.c_str(), 0) != 1) {
