@@ -124,7 +124,8 @@ a2_gid_t RequestGroup::gidCounter_ = 0;
 
 RequestGroup::RequestGroup(const SharedHandle<Option>& option)
   : gid_(newGID()),
-    option_(new Option(*option.get())),
+    state_(STATE_WAITING),
+    option_(option),
     numConcurrentCommand_(option->getAsInt(PREF_SPLIT)),
     numStreamConnection_(0),
     numStreamCommand_(0),
@@ -167,10 +168,11 @@ RequestGroup::RequestGroup(const SharedHandle<Option>& option)
 
 RequestGroup::~RequestGroup() {}
 
-bool RequestGroup::isCheckIntegrityReady() const
+bool RequestGroup::isCheckIntegrityReady()
 {
   return option_->getAsBool(PREF_CHECK_INTEGRITY) &&
-    (downloadContext_->isChecksumVerificationAvailable() ||
+    ((downloadContext_->isChecksumVerificationAvailable() &&
+      downloadFinishedByFileLength()) ||
      downloadContext_->isPieceHashVerificationAvailable());
 }
 
@@ -232,7 +234,7 @@ SharedHandle<CheckIntegrityEntry> RequestGroup::createCheckIntegrityEntry()
     // infoFile exists.
     loadAndOpenFile(infoFile);
     checkEntry.reset(new StreamCheckIntegrityEntry(this));
-  } else if(infoFile->exists()) {
+  } else if(isPreLocalFileCheckEnabled() && infoFile->exists()) {
     loadAndOpenFile(infoFile);
     if(downloadFinished()) {
 #ifdef ENABLE_MESSAGE_DIGEST
@@ -247,17 +249,16 @@ SharedHandle<CheckIntegrityEntry> RequestGroup::createCheckIntegrityEntry()
         {
           downloadContext_->setChecksumVerified(true);
           A2_LOG_NOTICE(fmt(MSG_DOWNLOAD_ALREADY_COMPLETED,
-                            util::itos(gid_).c_str(),
-                            downloadContext_->getBasePath().c_str()));
+                            gid_, downloadContext_->getBasePath().c_str()));
         }
     } else {
       checkEntry.reset(new StreamCheckIntegrityEntry(this));
     }
-  } else if(downloadFinishedByFileLength()) {
-    pieceStorage_->markAllPiecesDone();
+  } else
 #ifdef ENABLE_MESSAGE_DIGEST
-    if(option_->getAsBool(PREF_CHECK_INTEGRITY) &&
+    if(downloadFinishedByFileLength() &&
        downloadContext_->isChecksumVerificationAvailable()) {
+      pieceStorage_->markAllPiecesDone();
       loadAndOpenFile(infoFile);
       ChecksumCheckIntegrityEntry* tempEntry =
         new ChecksumCheckIntegrityEntry(this);
@@ -266,15 +267,9 @@ SharedHandle<CheckIntegrityEntry> RequestGroup::createCheckIntegrityEntry()
     } else
 #endif // ENABLE_MESSAGE_DIGEST
       {
-        downloadContext_->setChecksumVerified(true);
-        A2_LOG_NOTICE(fmt(MSG_DOWNLOAD_ALREADY_COMPLETED,
-                          util::itos(gid_).c_str(),
-                          downloadContext_->getBasePath().c_str()));
+        loadAndOpenFile(infoFile);
+        checkEntry.reset(new StreamCheckIntegrityEntry(this));
       }
-  } else {
-    loadAndOpenFile(infoFile);
-    checkEntry.reset(new StreamCheckIntegrityEntry(this));
-  }
   return checkEntry;
 }
 
@@ -352,17 +347,18 @@ void RequestGroup::createInitialCommand
         (option_->getAsInt(PREF_BT_TRACKER_INTERVAL));
       btAnnounce->shuffleAnnounce();
       
-      assert(btRegistry->get(gid_).isNull());
+      assert(!btRegistry->get(gid_));
       btRegistry->put
-        (gid_, BtObject
-         (downloadContext_,
-          pieceStorage_,
-          peerStorage,
-          btAnnounce,
-          btRuntime,
-          (progressInfoFile ?
-           SharedHandle<BtProgressInfoFile>(progressInfoFile) :
-           progressInfoFile_)));
+        (gid_, SharedHandle<BtObject>
+         (new BtObject
+          (downloadContext_,
+           pieceStorage_,
+           peerStorage,
+           btAnnounce,
+           btRuntime,
+           (progressInfoFile ?
+            SharedHandle<BtProgressInfoFile>(progressInfoFile) :
+            progressInfoFile_))));
       if(metadataGetMode) {
         if(option_->getAsBool(PREF_ENABLE_DHT) ||
            (!e->getOption()->getAsBool(PREF_DISABLE_IPV6) &&
@@ -379,8 +375,8 @@ void RequestGroup::createInitialCommand
             e->addCommand(dhtCommands);
           }
         } else {
-          A2_LOG_NOTICE("For BitTorrent Magnet URI, enabling DHT is strongly"
-                        " recommended. See --enable-dht option.");
+          A2_LOG_NOTICE(_("For BitTorrent Magnet URI, enabling DHT is strongly"
+                          " recommended. See --enable-dht option."));
         }
 
         SharedHandle<CheckIntegrityEntry> entry
@@ -391,7 +387,7 @@ void RequestGroup::createInitialCommand
       }
       removeDefunctControlFile(progressInfoFile);
       {
-        uint64_t actualFileSize = pieceStorage_->getDiskAdaptor()->size();
+        int64_t actualFileSize = pieceStorage_->getDiskAdaptor()->size();
         if(actualFileSize == downloadContext_->getTotalLength()) {
           // First, make DiskAdaptor read-only mode to allow the
           // program to seed file in read-only media.
@@ -401,9 +397,9 @@ void RequestGroup::createInitialCommand
           // truncate the file to downloadContext_->getTotalLength()
           A2_LOG_DEBUG
             (fmt("File size not match. File is opened in writable"
-                 " mode. Expected:%s Actual:%s",
-                 util::uitos(downloadContext_->getTotalLength()).c_str(),
-                 util::uitos(actualFileSize).c_str()));
+                 " mode. Expected:%" PRId64 " Actual:%" PRId64 "",
+                 downloadContext_->getTotalLength(),
+                 actualFileSize));
         }
       }
       // Call Load, Save and file allocation command here
@@ -556,7 +552,7 @@ void RequestGroup::processCheckIntegrityEntry
  const SharedHandle<CheckIntegrityEntry>& entry,
  DownloadEngine* e)
 {
-  uint64_t actualFileSize = pieceStorage_->getDiskAdaptor()->size();
+  int64_t actualFileSize = pieceStorage_->getDiskAdaptor()->size();
   if(actualFileSize > downloadContext_->getTotalLength()) {
     entry->cutTrailingGarbage();
   }
@@ -684,14 +680,12 @@ void RequestGroup::adjustFilename
      option_->getAsBool(PREF_REMOVE_CONTROL_FILE) &&
      infoFile->exists()) {
     infoFile->removeFile();
-    A2_LOG_NOTICE(fmt("Removed control file for %s because it is requested by"
-                      " user.",
+    A2_LOG_NOTICE(fmt(_("Removed control file for %s because it is requested by"
+                        " user."),
                       infoFile->getFilename().c_str()));
   }
   if(infoFile->exists()) {
     // Use current filename
-  } else if(downloadFinishedByFileLength()) {
-    // File was downloaded already, no need to change file name.
   } else {
     File outfile(getFirstFilePath());    
     if(outfile.exists() && option_->getAsBool(PREF_CONTINUE) &&
@@ -772,14 +766,12 @@ void RequestGroup::shouldCancelDownloadForSafety()
         A2_LOG_NOTICE(fmt(MSG_FILE_RENAMED, getFirstFilePath().c_str()));
       } else {
         throw DOWNLOAD_FAILURE_EXCEPTION2
-          (fmt("File renaming failed: %s",
-               getFirstFilePath().c_str()),
+          (fmt("File renaming failed: %s", getFirstFilePath().c_str()),
            error_code::FILE_RENAMING_FAILED);
       }
     } else {
       throw DOWNLOAD_FAILURE_EXCEPTION2
-        (fmt(MSG_FILE_ALREADY_EXISTS,
-             getFirstFilePath().c_str()),
+        (fmt(MSG_FILE_ALREADY_EXISTS, getFirstFilePath().c_str()),
          error_code::FILE_ALREADY_EXISTS);
     }
   }
@@ -791,8 +783,9 @@ bool RequestGroup::tryAutoFileRenaming()
   if(filepath.empty()) {
     return false;
   }
-  for(unsigned int i = 1; i < 10000; ++i) {
-    File newfile(strconcat(filepath, A2STR::DOT_C, util::uitos(i)));
+  for(int i = 1; i < 10000; ++i) {
+    std::string newfilename = fmt("%s.%d", filepath.c_str(), i);
+    File newfile(newfilename);
     File ctrlfile(newfile.getPath()+DefaultBtProgressInfoFile::getSuffix());
     if(!newfile.exists() || (newfile.exists() && ctrlfile.exists())) {
       downloadContext_->getFirstFileEntry()->setPath(newfile.getPath());
@@ -810,7 +803,7 @@ void RequestGroup::createNextCommandWithAdj(std::vector<Command*>& commands,
     numCommand = 1+numAdj;
   } else {
     numCommand = std::min(downloadContext_->getNumPieces(),
-                          numConcurrentCommand_);
+                          static_cast<size_t>(numConcurrentCommand_));
     numCommand += numAdj;
   }
   if(numCommand > 0) {
@@ -832,8 +825,9 @@ void RequestGroup::createNextCommand(std::vector<Command*>& commands,
     if(numStreamCommand_ >= numConcurrentCommand_) {
       numCommand = 0;
     } else {
-      numCommand = std::min(downloadContext_->getNumPieces(),
-                            numConcurrentCommand_-numStreamCommand_);
+      numCommand =
+        std::min(downloadContext_->getNumPieces(),
+                 static_cast<size_t>(numConcurrentCommand_-numStreamCommand_));
     }
   }
   if(numCommand > 0) {
@@ -843,9 +837,9 @@ void RequestGroup::createNextCommand(std::vector<Command*>& commands,
 
 void RequestGroup::createNextCommand(std::vector<Command*>& commands,
                                      DownloadEngine* e,
-                                     unsigned int numCommand)
+                                     int numCommand)
 {
-  for(; numCommand--; ) {
+  for(; numCommand > 0; --numCommand) {
     Command* command = new CreateRequestCommand(e->newCUID(), this, e);
     commands.push_back(command);
   }
@@ -865,7 +859,7 @@ std::string RequestGroup::getFirstFilePath() const
   }
 }
 
-uint64_t RequestGroup::getTotalLength() const
+int64_t RequestGroup::getTotalLength() const
 {
   if(!pieceStorage_) {
     return 0;
@@ -878,7 +872,7 @@ uint64_t RequestGroup::getTotalLength() const
   }
 }
 
-uint64_t RequestGroup::getCompletedLength() const
+int64_t RequestGroup::getCompletedLength() const
 {
   if(!pieceStorage_) {
     return 0;
@@ -904,17 +898,15 @@ void RequestGroup::validateFilename(const std::string& expectedFilename,
   }
 }
 
-void RequestGroup::validateTotalLength(uint64_t expectedTotalLength,
-                                       uint64_t actualTotalLength) const
+void RequestGroup::validateTotalLength(int64_t expectedTotalLength,
+                                       int64_t actualTotalLength) const
 {
   if(expectedTotalLength <= 0) {
     return;
   }
   if(expectedTotalLength != actualTotalLength) {
     throw DL_ABORT_EX
-      (fmt(EX_SIZE_MISMATCH,
-           util::itos(expectedTotalLength, true).c_str(),
-           util::itos(actualTotalLength, true).c_str()));
+      (fmt(EX_SIZE_MISMATCH, expectedTotalLength, actualTotalLength));
   }
 }
 
@@ -923,7 +915,7 @@ void RequestGroup::validateFilename(const std::string& actualFilename) const
   validateFilename(downloadContext_->getFileEntries().front()->getBasename(), actualFilename);
 }
 
-void RequestGroup::validateTotalLength(uint64_t actualTotalLength) const
+void RequestGroup::validateTotalLength(int64_t actualTotalLength) const
 {
   validateTotalLength(getTotalLength(), actualTotalLength);
 }
@@ -948,9 +940,9 @@ void RequestGroup::decreaseStreamConnection()
   --numStreamConnection_;
 }
 
-unsigned int RequestGroup::getNumConnection() const
+int RequestGroup::getNumConnection() const
 {
-  unsigned int numConnection = numStreamConnection_;
+  int numConnection = numStreamConnection_;
 #ifdef ENABLE_BITTORRENT
   if(btRuntime_) {
     numConnection += btRuntime_->getConnections();
@@ -968,7 +960,7 @@ void RequestGroup::decreaseNumCommand()
 {
   --numCommand_;
   if(!numCommand_ && requestGroupMan_) {
-    A2_LOG_DEBUG(fmt("GID#%s - Request queue check", util::itos(gid_).c_str()));
+    A2_LOG_DEBUG(fmt("GID#%" PRId64 " - Request queue check", gid_));
     requestGroupMan_->requestQueueCheck();
   }
 }
@@ -1157,14 +1149,13 @@ void RequestGroup::setProgressInfoFile(const BtProgressInfoFileHandle& progressI
 bool RequestGroup::needsFileAllocation() const
 {
   return isFileAllocationEnabled() &&
-    (uint64_t)option_->getAsLLInt(PREF_NO_FILE_ALLOCATION_LIMIT) <= getTotalLength() &&
+    option_->getAsLLInt(PREF_NO_FILE_ALLOCATION_LIMIT) <= getTotalLength() &&
     !pieceStorage_->getDiskAdaptor()->fileAllocationIterator()->finished();
 }
 
 DownloadResultHandle RequestGroup::createDownloadResult() const
 {
-  A2_LOG_DEBUG(fmt("GID#%s - Creating DownloadResult.",
-                   util::itos(gid_).c_str()));
+  A2_LOG_DEBUG(fmt("GID#%" PRId64 " - Creating DownloadResult.", gid_));
   TransferStat st = calculateStat();
   SharedHandle<DownloadResult> res(new DownloadResult());
   res->gid = gid_;
@@ -1182,13 +1173,15 @@ DownloadResultHandle RequestGroup::createDownloadResult() const
   res->uploadLength = st.getAllTimeUploadLength();
   if(pieceStorage_) {
     if(pieceStorage_->getBitfieldLength() > 0) {
-      res->bitfieldStr = util::toHex(pieceStorage_->getBitfield(),
-                                     pieceStorage_->getBitfieldLength());
+      res->bitfield.assign(pieceStorage_->getBitfield(),
+                           pieceStorage_->getBitfield()+
+                           pieceStorage_->getBitfieldLength());
     }
   }
 #ifdef ENABLE_BITTORRENT
   if(downloadContext_->hasAttribute(bittorrent::BITTORRENT)) {
-    res->infoHashStr = bittorrent::getInfoHashString(downloadContext_);
+    const unsigned char* p = bittorrent::getInfoHash(downloadContext_);
+    res->infoHash.assign(p, p+INFO_HASH_LENGTH);
   }
 #endif // ENABLE_BITTORRENT
   res->pieceLength = downloadContext_->getPieceLength();
@@ -1205,7 +1198,7 @@ void RequestGroup::reportDownloadFinished()
 #ifdef ENABLE_BITTORRENT
   if(downloadContext_->hasAttribute(bittorrent::BITTORRENT)) {
     TransferStat stat = calculateStat();
-    uint64_t completedLength = getCompletedLength();
+    int64_t completedLength = getCompletedLength();
     double shareRatio = completedLength == 0 ? 0.0 :
       1.0*stat.getAllTimeUploadLength()/completedLength;
     SharedHandle<TorrentAttribute> attrs =
@@ -1260,12 +1253,12 @@ void RequestGroup::updateLastModifiedTime(const Time& time)
 void RequestGroup::increaseAndValidateFileNotFoundCount()
 {
   ++fileNotFoundCount_;
-  const unsigned int maxCount = option_->getAsInt(PREF_MAX_FILE_NOT_FOUND);
+  const int maxCount = option_->getAsInt(PREF_MAX_FILE_NOT_FOUND);
   if(maxCount > 0 && fileNotFoundCount_ >= maxCount &&
      (!segmentMan_ ||
       segmentMan_->calculateSessionDownloadLength() == 0)) {
     throw DOWNLOAD_FAILURE_EXCEPTION2
-      (fmt("Reached max-file-not-found count=%u", maxCount),
+      (fmt("Reached max-file-not-found count=%d", maxCount),
        error_code::MAX_FILE_NOT_FOUND);
   }
 }
