@@ -94,7 +94,7 @@ void SegmentMan::init()
   // PieceStorage here?
 }
 
-uint64_t SegmentMan::getTotalLength() const
+int64_t SegmentMan::getTotalLength() const
 {
   if(!pieceStorage_) {
     return 0;
@@ -120,7 +120,7 @@ SharedHandle<Segment> SegmentMan::checkoutSegment
   if(!piece) {
     return SharedHandle<Segment>();
   }
-  A2_LOG_DEBUG(fmt("Attach segment#%lu to CUID#%lld.",
+  A2_LOG_DEBUG(fmt("Attach segment#%lu to CUID#%" PRId64 ".",
                    static_cast<unsigned long>(piece->getIndex()),
                    cuid));
   piece->setUsedBySegment(true);
@@ -132,21 +132,19 @@ SharedHandle<Segment> SegmentMan::checkoutSegment
   }
   SegmentEntryHandle entry(new SegmentEntry(cuid, segment));
   usedSegmentEntries_.push_back(entry);
-  A2_LOG_DEBUG(fmt("index=%lu, length=%lu, segmentLength=%lu,"
-                   " writtenLength=%lu",
+  A2_LOG_DEBUG(fmt("index=%lu, length=%d, segmentLength=%d,"
+                   " writtenLength=%d",
                    static_cast<unsigned long>(segment->getIndex()),
-                   static_cast<unsigned long>(segment->getLength()),
-                   static_cast<unsigned long>(segment->getSegmentLength()),
-                   static_cast<unsigned long>(segment->getWrittenLength())));
+                   segment->getLength(),
+                   segment->getSegmentLength(),
+                   segment->getWrittenLength()));
   if(piece->getLength() > 0) {
-    std::map<size_t, size_t>::iterator positr =
+    std::map<size_t, int32_t>::iterator positr =
       segmentWrittenLengthMemo_.find(segment->getIndex());
     if(positr != segmentWrittenLengthMemo_.end()) {
-      const size_t writtenLength = (*positr).second;
-      A2_LOG_DEBUG(fmt("writtenLength(in memo)=%lu, writtenLength=%lu",
-                       static_cast<unsigned long>(writtenLength),
-                       static_cast<unsigned long>(segment->getWrittenLength()))
-                   );
+      const int32_t writtenLength = (*positr).second;
+      A2_LOG_DEBUG(fmt("writtenLength(in memo)=%d, writtenLength=%d",
+                       writtenLength, segment->getWrittenLength()));
       //  If the difference between cached writtenLength and segment's
       //  writtenLength is less than one block, we assume that these
       //  missing bytes are already downloaded.
@@ -261,9 +259,9 @@ void SegmentMan::cancelSegmentInternal
   segment->getPiece()->setUsedBySegment(false);
   pieceStorage_->cancelPiece(segment->getPiece(), cuid);
   segmentWrittenLengthMemo_[segment->getIndex()] = segment->getWrittenLength();
-  A2_LOG_DEBUG(fmt("Memorized segment index=%lu, writtenLength=%lu",
+  A2_LOG_DEBUG(fmt("Memorized segment index=%lu, writtenLength=%d",
                    static_cast<unsigned long>(segment->getIndex()),
-                   static_cast<unsigned long>(segment->getWrittenLength())));
+                   segment->getWrittenLength()));
 }
 
 void SegmentMan::cancelSegment(cuid_t cuid) {
@@ -342,7 +340,7 @@ bool SegmentMan::hasSegment(size_t index) const {
   return pieceStorage_->hasPiece(index);
 }
 
-uint64_t SegmentMan::getDownloadLength() const {
+int64_t SegmentMan::getDownloadLength() const {
   if(!pieceStorage_) {
     return 0;
   } else {
@@ -406,17 +404,17 @@ void SegmentMan::updateFastestPeerStat(const SharedHandle<PeerStat>& peerStat)
   }
 }
 
-unsigned int SegmentMan::calculateDownloadSpeed()
+int SegmentMan::calculateDownloadSpeed()
 {
-  unsigned int speed = 0;
-  if(lastPeerStatDlspdMapUpdated_.differenceInMillis(global::wallclock)+
+  int speed = 0;
+  if(lastPeerStatDlspdMapUpdated_.differenceInMillis(global::wallclock())+
      A2_DELTA_MILLIS >= 250){
-    lastPeerStatDlspdMapUpdated_ = global::wallclock;
+    lastPeerStatDlspdMapUpdated_ = global::wallclock();
     peerStatDlspdMap_.clear();
     for(std::vector<SharedHandle<PeerStat> >::const_iterator i =
           peerStats_.begin(), eoi = peerStats_.end(); i != eoi; ++i) {
       if((*i)->getStatus() == PeerStat::ACTIVE) {
-        unsigned int s = (*i)->calculateDownloadSpeed();
+        int s = (*i)->calculateDownloadSpeed();
         peerStatDlspdMap_[(*i)->getCuid()] = s;
         speed += s;
       }
@@ -430,8 +428,8 @@ unsigned int SegmentMan::calculateDownloadSpeed()
 
 void SegmentMan::updateDownloadSpeedFor(const SharedHandle<PeerStat>& pstat)
 {
-  unsigned int newspd = pstat->calculateDownloadSpeed();
-  unsigned int oldSpd = peerStatDlspdMap_[pstat->getCuid()];
+  int newspd = pstat->calculateDownloadSpeed();
+  int oldSpd = peerStatDlspdMap_[pstat->getCuid()];
   if(cachedDlspd_ > oldSpd) {
     cachedDlspd_ -= oldSpd;
     cachedDlspd_ += newspd;
@@ -444,14 +442,14 @@ void SegmentMan::updateDownloadSpeedFor(const SharedHandle<PeerStat>& pstat)
 namespace {
 class PeerStatDownloadLengthOperator {
 public:
-  uint64_t operator()(uint64_t total, const SharedHandle<PeerStat>& ps)
+  int64_t operator()(int64_t total, const SharedHandle<PeerStat>& ps)
   {
     return ps->getSessionDownloadLength()+total;
   }
 };
 } // namespace
 
-uint64_t SegmentMan::calculateSessionDownloadLength() const
+int64_t SegmentMan::calculateSessionDownloadLength() const
 {
   return std::accumulate(fastestPeerStats_.begin(), fastestPeerStats_.end(),
                          0LL, PeerStatDownloadLengthOperator());
@@ -470,10 +468,11 @@ size_t SegmentMan::countFreePieceFrom(size_t index) const
 
 void SegmentMan::ignoreSegmentFor(const SharedHandle<FileEntry>& fileEntry)
 {
-  A2_LOG_DEBUG(fmt("ignoring segment for path=%s, offset=%s, length=%s",
+  A2_LOG_DEBUG(fmt("ignoring segment for path=%s, offset=%" PRId64
+                   ", length=%" PRId64 "",
                    fileEntry->getPath().c_str(),
-                   util::itos(fileEntry->getOffset()).c_str(),
-                   util::uitos(fileEntry->getLength()).c_str()));
+                   fileEntry->getOffset(),
+                   fileEntry->getLength()));
   ignoreBitfield_.addFilter(fileEntry->getOffset(), fileEntry->getLength());
 }
 
