@@ -65,11 +65,16 @@ std::string::const_iterator getNextDigit
 }
 } // namespace
 
-bool parseDate(time_t& time, const std::string& cookieDate)
+bool parseDate
+(time_t& time,
+ std::string::const_iterator first,
+ std::string::const_iterator last)
 {
+  // Following algorithm is described in
+  // http://tools.ietf.org/html/rfc6265#section-5.1.1
   std::vector<std::string> dateTokens;
-  for(std::string::const_iterator i = cookieDate.begin(),
-        eoi = cookieDate.end(); i != eoi;) {
+  for(std::string::const_iterator i = first,
+        eoi = last; i != eoi;) {
     unsigned char c = *i;
     if(isDelimiter(c)) {
       ++i;
@@ -129,17 +134,23 @@ bool parseDate(time_t& time, const std::string& cookieDate)
       }
     }
     if(!foundMonth) {
-      static std::string MONTH[] = {
+      static const char MONTH[][12] = {
         "jan", "feb", "mar", "apr",
         "may", "jun", "jul", "aug",
         "sep", "oct", "nov", "dec" };
       if((*i).size() >= 3) {
-        std::string head = (*i).substr(0, 3);
-        util::lowercase(head);
-        std::string* mptr = std::find(vbegin(MONTH), vend(MONTH), head);
-        if(mptr != vend(MONTH)) {
+        bool found = false;
+        size_t j;
+        for(j = 0; j < 12; ++j) {
+          if(util::strieq((*i).begin(), (*i).begin()+3,
+                          &MONTH[j][0], &MONTH[j][3])) {
+            found = true;
+            break;
+          }
+        }
+        if(found) {
           foundMonth = true;
-          month = std::distance(vbegin(MONTH), mptr)+1;
+          month = j+1;
           continue;
         }
       }
@@ -198,6 +209,8 @@ bool parse
  const std::string& defaultPath,
  time_t creationTime)
 {
+  // This implementation is based on the algorithm listed in
+  // http://tools.ietf.org/html/rfc6265
   std::string::const_iterator nvEnd = cookieStr.begin();
   std::string::const_iterator end = cookieStr.end();
   for(; nvEnd != end && *nvEnd != ';'; ++nvEnd);
@@ -206,11 +219,16 @@ bool parse
   if(eq == nvEnd) {
     return false;
   }
-  std::string cookieName = util::stripIter(cookieStr.begin(), eq);
-  if(cookieName.empty()) {
+  std::pair<std::string::const_iterator,
+            std::string::const_iterator> p =
+    util::stripIter(cookieStr.begin(), eq);
+  if(p.first == p.second) {
     return false;
   }
-  std::string cookieValue = util::stripIter(eq+1, nvEnd);
+  Scip cookieName(p.first, p.second);
+  p = util::stripIter(eq+1, nvEnd);
+  p = util::stripIter(p.first, p.second, "\"");
+  Scip cookieValue(p.first, p.second);
   time_t expiryTime = 0;
   bool foundExpires = false;
   bool persistent = false;
@@ -228,36 +246,39 @@ bool parse
   for(std::string::const_iterator i = nvEnd; i != end;) {
     std::string::const_iterator j = std::find(i, end, ';');
     std::string::const_iterator eq = std::find(i, j, '=');
-    std::string attrName = util::stripIter(i, eq);
-    util::lowercase(attrName);
-    std::string attrValue;
-    if(eq != j) {
-      attrValue = util::stripIter(eq+1, j);
+    p = util::stripIter(i, eq);
+    std::pair<std::string::const_iterator,
+              std::string::const_iterator> attrp;
+    if(eq == j) {
+      attrp.first = attrp.second = j;
+    } else {
+      attrp = util::stripIter(eq+1, j);
     }
     i = j;
     if(j != end) {
       ++i;
     }
-    if(attrName == "expires") {
-      if(parseDate(expiryTime, attrValue)) {
+    if(util::strieq(p.first, p.second, "expires")) {
+      if(parseDate(expiryTime, attrp.first, attrp.second)) {
         foundExpires = true;
       } else {
         return false;
       }
-    } else if(attrName == "max-age") {
-      if(attrValue.empty() ||
-         (!in(static_cast<unsigned char>(attrValue[0]), 0x30u, 0x39u) &&
-          attrValue[0] != '-')) {
+    } else if(util::strieq(p.first, p.second, "max-age")) {
+      if(attrp.first == attrp.second ||
+         (!in(static_cast<unsigned char>(*attrp.first), 0x30u, 0x39u) &&
+          *attrp.first != '-')) {
         return false;
       }
-      for(std::string::const_iterator s = attrValue.begin()+1,
-            eos = attrValue.end(); s != eos; ++s) {
+      for(std::string::const_iterator s = attrp.first+1,
+            eos = attrp.second; s != eos; ++s) {
         if(!in(static_cast<unsigned char>(*s), 0x30u, 0x39u)) {
           return false;
         }
       }
       int64_t delta;
-      if(util::parseLLIntNoThrow(delta, attrValue)) {
+      if(util::parseLLIntNoThrow(delta,
+                                 std::string(attrp.first, attrp.second))) {
         foundMaxAge = true;
         if(delta <= 0) {
           maxAge = 0;
@@ -273,26 +294,26 @@ bool parse
       } else {
         return false;
       }
-    } else if(attrName == "domain") {
-      if(attrValue.empty()) {
+    } else if(util::strieq(p.first, p.second, "domain")) {
+      if(attrp.first == attrp.second) {
         return false;
       }
-      std::string::const_iterator noDot = attrValue.begin();
-      std::string::const_iterator end = attrValue.end();
+      std::string::const_iterator noDot = attrp.first;
+      std::string::const_iterator end = attrp.second;
       for(; noDot != end && *noDot == '.'; ++noDot);
       if(noDot == end) {
         return false;
       }
-      cookieDomain = std::string(noDot, end);
-    } else if(attrName == "path") {
-      if(goodPath(attrValue)) {
-        cookiePath = attrValue;
+      cookieDomain.assign(noDot, end);
+    } else if(util::strieq(p.first, p.second, "path")) {
+      if(goodPath(attrp.first, attrp.second)) {
+        cookiePath.assign(attrp.first, attrp.second);
       } else {
         cookiePath = defaultPath;
       }
-    } else if(attrName == "secure") {
+    } else if(util::strieq(p.first, p.second, "secure")) {
       secure = true;
-    } else if(attrName == "httponly") {
+    } else if(util::strieq(p.first, p.second, "httponly")) {
       httpOnly = true;
     }
   }
@@ -321,8 +342,8 @@ bool parse
     cookiePath = defaultPath;
   }
   
-  cookie.setName(cookieName);
-  cookie.setValue(cookieValue);
+  cookie.setName(cookieName.first, cookieName.second);
+  cookie.setValue(cookieValue.first, cookieValue.second);
   cookie.setExpiryTime(expiryTime);
   cookie.setPersistent(persistent);
   cookie.setDomain(cookieDomain);
@@ -336,17 +357,11 @@ bool parse
   return true;
 }
 
-std::string removePrecedingDots(const std::string& host)
+bool goodPath
+(std::string::const_iterator first,
+ std::string::const_iterator last)
 {
-  std::string::const_iterator noDot = host.begin();
-  std::string::const_iterator end = host.end();
-  for(; noDot != end && *noDot == '.'; ++noDot);
-  return std::string(noDot, end);
-}
-
-bool goodPath(const std::string& cookiePath)
-{
-  return !cookiePath.empty() && cookiePath[0] == '/';
+  return first != last && *first == '/';
 }
 
 std::string canonicalizeHost(const std::string& host)
