@@ -37,6 +37,7 @@ class HttpRequestTest : public CppUnit::TestFixture {
   CPPUNIT_TEST(testAddHeader);
   CPPUNIT_TEST(testAddAcceptType);
   CPPUNIT_TEST(testEnableAcceptEncoding);
+  CPPUNIT_TEST(testConditionalRequest);
   CPPUNIT_TEST_SUITE_END();
 private:
   SharedHandle<Option> option_;
@@ -64,6 +65,7 @@ public:
   void testAddHeader();
   void testAddAcceptType();
   void testEnableAcceptEncoding();
+  void testConditionalRequest();
 };
 
 
@@ -76,12 +78,12 @@ void HttpRequestTest::testGetStartByte()
   SharedHandle<Segment> segment(new PiecedSegment(1024, p));
   SharedHandle<FileEntry> fileEntry(new FileEntry("file", 1024*10, 0));
 
-  CPPUNIT_ASSERT_EQUAL((off_t)0LL, httpRequest.getStartByte());
+  CPPUNIT_ASSERT_EQUAL((int64_t)0LL, httpRequest.getStartByte());
 
   httpRequest.setSegment(segment);
   httpRequest.setFileEntry(fileEntry);
   
-  CPPUNIT_ASSERT_EQUAL((off_t)1024LL, httpRequest.getStartByte());
+  CPPUNIT_ASSERT_EQUAL((int64_t)1024LL, httpRequest.getStartByte());
 }
 
 void HttpRequestTest::testGetEndByte()
@@ -95,11 +97,11 @@ void HttpRequestTest::testGetEndByte()
   SharedHandle<Segment> segment(new PiecedSegment(segmentLength, piece));
   SharedHandle<FileEntry> fileEntry(new FileEntry("file", segmentLength*10, 0));
 
-  CPPUNIT_ASSERT_EQUAL((off_t)0LL, httpRequest.getEndByte());
+  CPPUNIT_ASSERT_EQUAL((int64_t)0LL, httpRequest.getEndByte());
 
   httpRequest.setSegment(segment);
 
-  CPPUNIT_ASSERT_EQUAL((off_t)0LL, httpRequest.getEndByte());
+  CPPUNIT_ASSERT_EQUAL((int64_t)0LL, httpRequest.getEndByte());
 
   SharedHandle<Request> request(new Request());
   request->supportsPersistentConnection(true);
@@ -108,18 +110,18 @@ void HttpRequestTest::testGetEndByte()
   httpRequest.setRequest(request);
   httpRequest.setFileEntry(fileEntry);
 
-  CPPUNIT_ASSERT_EQUAL((off_t)(segmentLength*index+length-1),
+  CPPUNIT_ASSERT_EQUAL((int64_t)(segmentLength*index+length-1),
                        httpRequest.getEndByte());
 
   // The end byte of FileEntry are placed inside segment
   fileEntry->setLength(segmentLength+100);
 
-  CPPUNIT_ASSERT_EQUAL((off_t)(segmentLength*index+100-1),
+  CPPUNIT_ASSERT_EQUAL((int64_t)(segmentLength*index+100-1),
                        httpRequest.getEndByte());
 
   request->setPipeliningHint(false);
 
-  CPPUNIT_ASSERT_EQUAL((off_t)0LL, httpRequest.getEndByte());
+  CPPUNIT_ASSERT_EQUAL((int64_t)0LL, httpRequest.getEndByte());
 }
 
 void HttpRequestTest::testCreateRequest()
@@ -242,7 +244,7 @@ void HttpRequestTest::testCreateRequest()
   option_->put(PREF_HTTP_PASSWD, "aria2passwd");
 
   CPPUNIT_ASSERT(authConfigFactory_->activateBasicCred
-                 ("localhost", "/", option_.get()));
+                 ("localhost", 8080, "/", option_.get()));
 
   expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -793,9 +795,9 @@ void HttpRequestTest::testEnableAcceptEncoding()
   httpRequest.setAuthConfigFactory(authConfigFactory_, option_.get());
 
   std::string acceptEncodings;
-#ifdef HAVE_LIBZ
+#ifdef HAVE_ZLIB
   acceptEncodings += "deflate, gzip";
-#endif // HAVE_LIBZ
+#endif // HAVE_ZLIB
   
   std::string expectedTextHead =
     "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
@@ -843,4 +845,19 @@ void HttpRequestTest::testCreateRequest_ipv6LiteralAddr()
   CPPUNIT_ASSERT(proxyRequest.find("CONNECT [::1]:80 ") != std::string::npos);
 }
 
+void HttpRequestTest::testConditionalRequest()
+{
+  HttpRequest httpRequest;
+  CPPUNIT_ASSERT(!httpRequest.conditionalRequest());
+  httpRequest.setIfModifiedSinceHeader("dummy");
+  CPPUNIT_ASSERT(httpRequest.conditionalRequest());
+  httpRequest.setIfModifiedSinceHeader("");
+  CPPUNIT_ASSERT(!httpRequest.conditionalRequest());
+  httpRequest.addHeader("If-None-Match: *");
+  CPPUNIT_ASSERT(httpRequest.conditionalRequest());
+  httpRequest.clearHeader();
+  httpRequest.addHeader("If-Modified-Since: dummy");
+  CPPUNIT_ASSERT(httpRequest.conditionalRequest());
+}
+
 } // namespace aria2
